// Code generated by smithy-go-codegen DO NOT EDIT.

package macie2

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/aws-sdk-go-v2/service/macie2/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Retrieves a subset of information about the classification scope for an account.
func (c *Client) ListClassificationScopes(ctx context.Context, params *ListClassificationScopesInput, optFns ...func(*Options)) (*ListClassificationScopesOutput, error) {
	if params == nil {
		params = &ListClassificationScopesInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListClassificationScopes", params, optFns, c.addOperationListClassificationScopesMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListClassificationScopesOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListClassificationScopesInput struct {

	// The name of the classification scope to retrieve the unique identifier for.
	Name *string

	// The nextToken string that specifies which page of results to return in a
	// paginated response.
	NextToken *string

	noSmithyDocumentSerde
}

type ListClassificationScopesOutput struct {

	// An array that specifies the unique identifier and name of the classification
	// scope for the account.
	ClassificationScopes []types.ClassificationScopeSummary

	// The string to use in a subsequent request to get the next page of results in a
	// paginated response. This value is null if there are no additional pages.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListClassificationScopesMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpListClassificationScopes{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpListClassificationScopes{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "ListClassificationScopes"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListClassificationScopes(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

// ListClassificationScopesAPIClient is a client that implements the
// ListClassificationScopes operation.
type ListClassificationScopesAPIClient interface {
	ListClassificationScopes(context.Context, *ListClassificationScopesInput, ...func(*Options)) (*ListClassificationScopesOutput, error)
}

var _ ListClassificationScopesAPIClient = (*Client)(nil)

// ListClassificationScopesPaginatorOptions is the paginator options for
// ListClassificationScopes
type ListClassificationScopesPaginatorOptions struct {
	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListClassificationScopesPaginator is a paginator for ListClassificationScopes
type ListClassificationScopesPaginator struct {
	options   ListClassificationScopesPaginatorOptions
	client    ListClassificationScopesAPIClient
	params    *ListClassificationScopesInput
	nextToken *string
	firstPage bool
}

// NewListClassificationScopesPaginator returns a new
// ListClassificationScopesPaginator
func NewListClassificationScopesPaginator(client ListClassificationScopesAPIClient, params *ListClassificationScopesInput, optFns ...func(*ListClassificationScopesPaginatorOptions)) *ListClassificationScopesPaginator {
	if params == nil {
		params = &ListClassificationScopesInput{}
	}

	options := ListClassificationScopesPaginatorOptions{}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListClassificationScopesPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListClassificationScopesPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListClassificationScopes page.
func (p *ListClassificationScopesPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListClassificationScopesOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	result, err := p.client.ListClassificationScopes(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

func newServiceMetadataMiddleware_opListClassificationScopes(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "ListClassificationScopes",
	}
}
