// Code generated by smithy-go-codegen DO NOT EDIT.

package workmail

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/aws-sdk-go-v2/service/workmail/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Lists all the impersonation roles for the given WorkMail organization.
func (c *Client) ListImpersonationRoles(ctx context.Context, params *ListImpersonationRolesInput, optFns ...func(*Options)) (*ListImpersonationRolesOutput, error) {
	if params == nil {
		params = &ListImpersonationRolesInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListImpersonationRoles", params, optFns, c.addOperationListImpersonationRolesMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListImpersonationRolesOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListImpersonationRolesInput struct {

	// The WorkMail organization to which the listed impersonation roles belong.
	//
	// This member is required.
	OrganizationId *string

	// The maximum number of results returned in a single call.
	MaxResults *int32

	// The token used to retrieve the next page of results. The first call doesn't
	// require a token.
	NextToken *string

	noSmithyDocumentSerde
}

type ListImpersonationRolesOutput struct {

	// The token to retrieve the next page of results. The value is null when there
	// are no results to return.
	NextToken *string

	// The list of impersonation roles under the given WorkMail organization.
	Roles []types.ImpersonationRole

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListImpersonationRolesMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpListImpersonationRoles{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpListImpersonationRoles{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "ListImpersonationRoles"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addOpListImpersonationRolesValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListImpersonationRoles(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

// ListImpersonationRolesAPIClient is a client that implements the
// ListImpersonationRoles operation.
type ListImpersonationRolesAPIClient interface {
	ListImpersonationRoles(context.Context, *ListImpersonationRolesInput, ...func(*Options)) (*ListImpersonationRolesOutput, error)
}

var _ ListImpersonationRolesAPIClient = (*Client)(nil)

// ListImpersonationRolesPaginatorOptions is the paginator options for
// ListImpersonationRoles
type ListImpersonationRolesPaginatorOptions struct {
	// The maximum number of results returned in a single call.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListImpersonationRolesPaginator is a paginator for ListImpersonationRoles
type ListImpersonationRolesPaginator struct {
	options   ListImpersonationRolesPaginatorOptions
	client    ListImpersonationRolesAPIClient
	params    *ListImpersonationRolesInput
	nextToken *string
	firstPage bool
}

// NewListImpersonationRolesPaginator returns a new ListImpersonationRolesPaginator
func NewListImpersonationRolesPaginator(client ListImpersonationRolesAPIClient, params *ListImpersonationRolesInput, optFns ...func(*ListImpersonationRolesPaginatorOptions)) *ListImpersonationRolesPaginator {
	if params == nil {
		params = &ListImpersonationRolesInput{}
	}

	options := ListImpersonationRolesPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListImpersonationRolesPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListImpersonationRolesPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListImpersonationRoles page.
func (p *ListImpersonationRolesPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListImpersonationRolesOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	result, err := p.client.ListImpersonationRoles(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

func newServiceMetadataMiddleware_opListImpersonationRoles(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "ListImpersonationRoles",
	}
}
